#!/usr/bin/perl -I /usr/local/accounts/lib


package RatsClientLib;
require 5.001;

use lib "/usr/local/accounts/lib";
require "/usr/local/accounts/etc/rats_internal.conf";
use RATSdes;
use Socket;
use Sys::Hostname;
use IO::Socket;
use Carp;
use File::Basename;
require Exporter;
@ISA = qw(Exporter);



# we define the log routine for all to see right here
my $log = sub {
    &{$CONFIG::COMMAND_HOOKS{'LOG'}};
};
# same for debug
my $debug = sub {
    &{$CONFIG::COMMAND_HOOKS{'DEBUG'}};
};

# if the verbose switch is on then print the message
sub blather
{
    print @_ if ($main::VERB && !$main::QUIET);
}

# if the quiet switch is off then print the message
sub say
{
    print @_ unless ($main::QUIET);
}


# main logging routine. 
sub default_log (@){
    my ($remote, @msg_parts, $msg);
    if ($_[0] eq 'REMOTE') {
	($remote, @msg_parts) = @_;
    } else {
	@msg_parts = @_;
    } 
    $msg = join(":", @msg_parts);
    chomp $msg;
    my $host = &get_hostname();
    my $target_host = $CONFIG::LOG_HOST;

    # cause I'm to lazy to do a printf the size of vars may be 1 or 2 digits
    my $name = basename $0;
    
    unless ($remote eq "REMOTE") {
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	# $mon is reported in 0->11 so we add 1 for human consumption
	$mon++;
	$year += 1900;
	my $calling = (caller(2))[3];
	$msg = "$mon/$mday/$year $hour:$min:$sec $host:$name:$$:$calling: $msg";
    }
    if ($host eq $target_host) {
	my $handler_sub = $CONFIG::COMMAND_HOOKS{'LLOG'};
	return &$handler_sub($msg);
    } else {
	my $handler_sub = $CONFIG::COMMAND_HOOKS{'RLOG'};
	return &$handler_sub($msg);
    }
}

# remote logging routine.  This does its own remote connection
# to avoid recursive errors
sub default_remote_log (@) {
    
    return &send_remote_command("LOG", "REMOTE", $_[0],  $CONFIG::LOG_HOST, $CONFIG::CLIENT_PORT);
}



# logging routine for local stuff
sub default_local_log ( $ ) {
    my ($msg) = @_;
    unless (open (LOG, ">>$CONFIG::LOG_FILE")) {
        return "$CONFIG::API{'CODEDERR'}:$CONFIG::RESPONSE{'FOL'}";
    }
    print LOG "$msg\n";
    close LOG;
    return "$CONFIG::API{'R_LOG'}:$CONFIG::RESPONSE{'SCS'}";
}

# extra loging routine driven by a variable defined in main
sub default_debug ($) {
    if ($main::DEBUG) {
	
	my $host = &get_hostname();
	my $target_host = $CONFIG::LOG_HOST;
	
	# cause I'm to lazy to do a printf the size of vars may be 1 or 2 digits
	my $name = basename $0;
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
	# $mon is reported in 0->11 so we add 1 for human consumption
	$mon++;
	$year += 1900;
	my $calling = (caller(2))[3];
	my $msg = "$mon/$mday/$year $hour:$min:$sec $host:$name:$$:$calling: $_[0]";
	&$log("REMOTE", $msg);
    }
}



# logging die routine
sub dye( $ ) {
    my $msg = $_[0];
    &$log($msg);
    &$log("exiting ...\n");
    die $msg;
}


# takes a number of args, uses the last one as a hostname and sends a 
# encrypted form of the rest of the args to that host. Returns 1 if
# succesfull, -1 if connect failed, 0 otherwise.
sub send_remote_command (@) {
    use Socket;
    my (@args) = @_;
    my ($response,$more_data,$code,@data,@buffer,$complex);
    my $port = pop @args;
    my $host = pop @args;
    my $hostname = &get_hostname();
    # get my local keys
    my ($key1, $key2) = split (":", $CONFIG::DES_KEYS{$hostname});
    
    # setup crypto
    my $encrypt_sub = $CONFIG::INTERNAL_FUNCTIONS{'ENCRYPT'};
    my $decrypt_sub = $CONFIG::INTERNAL_FUNCTIONS{'DECRYPT'};


    # setup message    
    my $command = shift @args;
    $command = $CONFIG::API{$command};
    
    
    my $message = join(":", $command, @args) . ":";

    my $size;
    if (length($message) % 8) {
	$size = length($message) + 8 - (length($message) % 8);
    } else {
	$size = length($message);
    }
    # encrypt message
    my $encrypted_message = &$encrypt_sub($message, $key1, $key2,$size);

    #get size of message
    $size = pack ('N', $size);
    # check host and untaint
    if ($host =~ /^([-\w.]+)$/) {
        $host = $1;                     # $host now untainted
    } else {
	return "$CONFIG::API{'CODEDERR'}:$CONFIG::RESPONSE{'TNT'}";
    }
    # check port and untaint
    if ($port =~ /^(\d+)$/) {
        $port = $1;                     # $port now untainted
    } else {
	return "$CONFIG::API{'CODEDERR'}:$CONFIG::RESPONSE{'TNT'}";
    }
    

    # setup connection
    my $iaddr = inet_aton($host);
    my $paddr = sockaddr_in($port, $iaddr);
    my $proto = getprotobyname('tcp');
    socket(SOCK, PF_INET, SOCK_STREAM, $proto) || return "$CONFIG::API{'CODEDERR'}:$CONFIG::RESPONSE{'SCF'}";
    connect (SOCK, $paddr) ||  return "$CONFIG::API{'CODEDERR'}:$CONFIG::RESPONSE{'SCF'}";

    # send command
    send SOCK, "$size$encrypted_message","";
    # note that if api code ends in one we read until its not.
    # this allows for calls that return more then one answer.
    $more_data = 1;
    $complex = 0;
    eval {

	local $SIG{ALRM} = sub { die "alarm\n" };
	alarm($CONFIG::NET_READ_TIMEOUT);
	while ($more_data) {
	    read SOCK, $size, $CONFIG::PKT_SIZE;
	    if ($size eq "") {
		close(SOCK);
		return "$CONFIG::API{'CODEDERR'}:$CONFIG::RESPONSE{'SRF'}";
	    }
	    $size = unpack('N', $size);
	    read SOCK, $response, $size;
	    $response = &$decrypt_sub($response, $key1, $key2, $size);
	    ($code, @data) = split(":",$response);
	    push (@buffer, @data);
	    if($code =~ /1$/) {
		$complex = 1;
	    } else {
		$more_data=0;
	    }
	}
	alarm(0);
    };
    # get results

    # catch the wiered read timeout
    if ($@) {
	return "$CONFIG::API{'CODEDERR'}:$CONFIG::RESPONSE{'SRT'}";
    }
    if ($complex) {
	$response = join (":", $code, @buffer);
    }
    shutdown SOCK, "2";
    return $response;    
}


# try to figure out on what filesystem a path would actualy reside
sub determine_fs ($) {
    my $path = $_[0];
    $path =~ s/\/$//;     # strip ending "/" if it exists
    &$debug("Got $path\n");
    my @parts = split("/", $path);
    while ( $#parts > 0 ) {
	#build the path
	$path = join("/", @parts);
	&$debug("Trying $path\n");
	#if an alias then return the real one
	if ($CONFIG::LINKED_FS{$path}) {
	    &$debug("found alias for $path\n");
	    return $CONFIG::LINKED_FS{$path};
	}
	#if defined then return it
	if ($CONFIG::FS{$path}) {
	    &$debug("found entry for $path\n");
	    return $path;
	}
	#if not reduce the path by one element and try again
	pop @parts;
    }
    # return a blank string if no match was found
    &$debug("failed to find info for $_[0]\n");
    return "";
}


# stub function used by the config file to set web locks
# if db lock is present.
sub config_db_lock_check() {
    if ($CONFIG::USE_CLIENT_DB) {	
	my $result = &send_remote_command("CHKDBLOCK",$CONFIG::MASTER_HOST,$CONFIG::CLIENT_PORT);
	my ($code,@data) = split(":", $result);
	if ($code == $CONFIG::API{'R_CHKDBLOCK'} && $data[0] == 0) {
	    return 0;
	} else {
	    return 1;
	}
    } else {
	return 0;
    }
}

#simple uudecode function
sub uudecode() {
    my ($string) =@_;

    my $block = unpack ("u", $string);

    return $block;
}
#simple uuencode function
sub uuencode() {
    my ($string) =@_;
    my $output;
    my $line;
    my $offset =0;
    print "uue in\n";
    while($line = substr($string,$offset,45)) {
	print "in loop $offset of ".length($string)."\n";
	$string =~ tr/$line//;
	$offset +=45;
	$output .= pack ("u", $line);
	chomp $output;
    }
    print "uue out\n";
    return $output;
}


sub get_hostname() {
    my $host = hostname();
    if ($CONFIG::FQDN_DOMAIN) {
	$host .= ".$CONFIG::FQDN_DOMAIN";
    }
    return $host;
}

1;

