#!/usr/bin/perl -T
use File::Copy;
use strict;
use File::Basename;
use CGI qw(:standard);
use vars qw(%States $page);
use vars qw($user $auth_user $log $debug);
use vars qw($do $c_port $s_port $server $master);

require "/usr/local/accounts/lib/RatsClientLib.pm";
require "/usr/local/accounts/etc/rats_internal.conf";
require "/usr/local/accounts/lib/RatsClientStyle.pm";

# we define the log routine for all to see right here
$log = $CONFIG::COMMAND_HOOKS{'LOG'};
# same for debug
$debug = $CONFIG::COMMAND_HOOKS{'DEBUG'};

# remote call function
$do = \&RatsClientLib::send_remote_command;
$c_port=$CONFIG::CLIENT_PORT;
$s_port=$CONFIG::SERVER_PORT;
$server = $CONFIG::MASTER_HOST;
$master = $CONFIG::RATS_MASTER;

## the user gets set by httpd from the AUTH part
$auth_user=$ENV{REMOTE_USER};

## create a new query and set the header
my $q = new CGI;
print $q->header;

my $title = $CONFIG::SETKERB_WEB_TITLE;
print $q->start_html(-title=>$title,
		     -style=>{'src'=>$RatsStyle::StyleSheetLocation},
		     -dtd=>'-//W3C//DTD HTML 4.0 Transitional//EN',
		    );
  
&RatsStyle::doc_frame_nomenu($title);
&RatsStyle::start_cgi();


## And if the user is not set then we quit
if ($auth_user eq "") {
    print error("Unauthenticated. Bye.");
    &RatsStyle::end_cgi();
    &RatsStyle::copyright_center();
    &RatsStyle::doc_end();
    exit -1;
}

# this gets the current state of the script
$page = $q->param(".State") || "Default";


## This allows for the many in one script. 
%States = (
           'Default' => \&main_screen,
           'Kerbshell'  => \&lock_user,
	   );


# we dispatch to the appropriate routine
if ($CONFIG::WEB_NO_SERVICE) {
    print &error($CONFIG::WEB_NO_SERVICE_MSG);
} else {
    if (exists $States{$page}) {
	$States{$page}->();
    } else {
	print error("No such page");
    }
}
&RatsStyle::end_cgi();
&RatsStyle::copyright_center();
&RatsStyle::doc_end();
exit;

# give a user field and a pull down menu of locks
sub main_screen {
    my @list = keys %CONFIG::SHELL_LOCKS;
    print
        "Hello, $auth_user.",p,
        $q->h1('Choose a User to Kerbshell'),p,
        $q->start_form,
	'User:',
	$q->textfield('USER'),p,
        to_page("Kerbshell"),
        $q->end_form;
    return;
}


sub lock_user {
    my ($code,@data,$username,$pass,$uid,$gid,$gcos,$home,$shell);
    my ($response,$new_shell,$timestamp);
	
    my $lock_type = 'kerbshell';
    my $username =  $q->param('USER');
    
    # verify that lock type is valid 
    unless ($CONFIG::SHELL_LOCKS{$lock_type}) {
	print error("Lock type $lock_type is not valid."),p;
	return
    }

    # check user data
    $response = &$do('GETPWENT', $username, $server,$c_port);
    ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_GETPWENT'}) {
	($username,$pass,$uid,$gid,$gcos,$home,$shell,$timestamp) = @data;
    } else {
	print error("No such user $username\n."),p;
	return &main_screen();
    }

    #check if the shell is a normal shell
    unless ($shell =~ /^$CONFIG::SHELL_LOCKS{'normal'}/) {
	print 
	    error("User does not have a normal shell.\n"),p,
	    error("A kerbshell can not be set for this user.\n"),p;
	return;
    }


    # build the new shell
    $shell = basename $shell;
    $new_shell = join("/",$CONFIG::SHELL_LOCKS{$lock_type},$shell);

    $response = &$do('CHGSHELL', $username, $new_shell, $timestamp, $server,$c_port);
    ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_CHGSHELL'}) { 
	print 
	    info("User $username locked with with lock type \"$lock_type\"\n"),
	    p;
	return &main_screen();
    } else {
	print error("Error setting lock: @data\n"),p;
	return &main_screen();
    }
}


# submit button with a hidden field.
# its only arg should be the value of the hidden state
sub to_page ($) {
    my $val = shift;
    return &RatsStyle::to_page($q,$val);
}
# format error message
sub error ($) {
  return &RatsStyle::error($_[0]);
}

# format info message
sub info ($) {
  return &RatsStyle::info($_[0]);
}
