#!/usr/bin/perl -T
use File::Copy;
use strict;
use File::Basename;
use CGI qw(:standard);
use vars qw(%States $page);
use vars qw($user $auth_user $log $debug);
use vars qw($do $c_port $s_port $server $master);

require "/usr/local/accounts/lib/RatsClientLib.pm";
require "/usr/local/accounts/etc/rats_internal.conf";
require "/usr/local/accounts/lib/RatsClientStyle.pm";

# we define the log routine for all to see right here
$log = $CONFIG::COMMAND_HOOKS{'LOG'};
# same for debug
$debug = $CONFIG::COMMAND_HOOKS{'DEBUG'};

# remote call function
$do = \&RatsClientLib::send_remote_command;
$c_port=$CONFIG::CLIENT_PORT;
$s_port=$CONFIG::SERVER_PORT;
$server = $CONFIG::MASTER_HOST;
$master = $CONFIG::RATS_MASTER;


## the user gets set by httpd from the AUTH part
$auth_user=$ENV{REMOTE_USER};

## create a new query and set the header
my $q = new CGI;
print $q->header;
my $title = $CONFIG::CHGGCOS_WEB_TITLE;
print $q->start_html(-title=>$title,
		       -style=>{'src'=>$RatsStyle::StyleSheetLocation},
		       -dtd=>'-//W3C//DTD HTML 4.0 Transitional//EN',
		      );
  
&RatsStyle::doc_frame_nomenu($title);
&RatsStyle::start_cgi();



## And if the user is not set then we quit
if ($auth_user eq "") {
    print error("Unauthenticated. Bye.");
    &RatsStyle::end_cgi();
    &RatsStyle::copyright_center();
    &RatsStyle::doc_end();
    exit -1;
}


# this gets the current state of the script
$page = $q->param(".State") || "Default";


## This allows for the many in one script. 
%States = (
           'Default' => \&main_screen,
           'Change'  => \&change_gcos,
	   );


# we dispatch to the appropriate routine
if ($CONFIG::WEB_NO_SERVICE) {
    print &error($CONFIG::WEB_NO_SERVICE_MSG);
} else {
    if (exists $States{$page}) {
	$States{$page}->();
    } else {
	print error("No such page");
    }
}

&RatsStyle::end_cgi();
&RatsStyle::copyright_center();
&RatsStyle::doc_end();



# give the user the change screen
sub main_screen {
    my ($username,$pass,$uid,$gid,$gcos,$home,$old_shell,$timestamp);

    my $max = $CONFIG::GCOS_FIELD_SIZE;

    my $response = &$do('GETPWENT', $auth_user, $server,$c_port);
    my ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_GETPWENT'}) {
	($username,$pass,$uid,$gid,$gcos,$home,$old_shell,$timestamp) = @data;
    } else {
	print error("No such user $auth_user\n."),p;
	return;
    }
    print
        $q->h1("Hello, $auth_user."),p,
	("Your current GCOS field is:"),
	info($gcos),p,
        $q->start_form,
	"Please enter new GCOS field: ",
        $q->textfield(-name=>'GCOS',
		     -default=>$gcos,
		     -size=>32
		     -maxlength=>$max),p,
	
        to_page("Change"),
        $q->end_form;
    return;
}


sub change_gcos {
    my ($code,@data,$username,$pass,$uid,$gid,$old_gcos,$home,$shell);
    my ($response,$gcos,$timestamp);
	
    my $gcos = $q->param('GCOS');

    #check gcos format
    unless ($gcos =~ /^($CONFIG::GCOS_REGEXP)$/) {
	print error("GCOS content not valid.\n");
	return &main_screen();
    }

    # check user data
    $response = &$do('GETPWENT', $auth_user, $server,$c_port);
    ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_GETPWENT'}) {
	($username,$pass,$uid,$gid,$old_gcos,$home,$shell,$timestamp) = @data;
    } else {
	print error("No such user $username\n."),p;
	return &main_screen();
    }

    # build the new shell
    $response = &$do('CHGGCOS', $username, $gcos, $timestamp, $server,$c_port);
    ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_CHGGCOS'}) { 
	print 
	    info("GCOS for $username changed to \"$gcos\"\n"),
	    p;
	return;
    } else {
	print error("Error setting new gcos: @data\n"),p;
	return &main_screen();
    }
}

# submit button with a hidden field.
# its only arg should be the value of the hidden state
sub to_page ($) {
    my $val = shift;
    return &RatsStyle::to_page($q,$val);
}
# format error message
sub error ($) {
  return &RatsStyle::error($_[0]);
}

# format info message
sub info ($) {
  return &RatsStyle::info($_[0]);
}
