#!/usr/bin/perl -T
use File::Basename;
use strict;
use CGI qw/:standard/;
# include config file

require "/usr/local/accounts/lib/RatsClientLib.pm";
require "/usr/local/accounts/etc/rats_internal.conf";
require "/usr/local/accounts/lib/RatsClientStyle.pm";

use vars qw(%States $page @groups $grname $gid $pass $members);
use vars qw($do $c_port $s_port $log $debug $DEBUG $pgm);
use vars qw($target_host $rats_master $timestamp $auth_user);
# we define the log routine for all to see right here
$log = $CONFIG::COMMAND_HOOKS{'LOG'};
# same for debug
$debug = $CONFIG::COMMAND_HOOKS{'DEBUG'};

$pgm = basename $0;

# remote call function
$do = \&RatsClientLib::send_remote_command;
$c_port=$CONFIG::CLIENT_PORT;
$s_port=$CONFIG::SERVER_PORT;
$target_host = $CONFIG::MASTER_HOST;
$rats_master = $CONFIG::RATS_MASTER;

## the user gets set by httpd from the AUTH part
$auth_user=$ENV{REMOTE_USER};

$DEBUG =0 ;
## create a new query
my $q = new CGI;

# set the header
print $q->header;
my $title = $CONFIG::CHGPASS_WEB_TITLE;
print $q->start_html(-title=>$title,
		       -style=>{'src'=>$RatsStyle::StyleSheetLocation},
		       -dtd=>'-//W3C//DTD HTML 4.0 Transitional//EN',
		      );
  
&RatsStyle::doc_frame_nomenu($title);
&RatsStyle::start_cgi();


## And if the user is not set then we quit
if ($auth_user eq "") {
    print error("Unauthenticated. Bye.");
    &RatsStyle::end_cgi();
    &RatsStyle::copyright_center();
    &RatsStyle::doc_end();
    exit -1;
}

# this gets the current state of the script
$page = $q->param(".State") ||  "Default";

## This allows for the many in one script. 
%States = (
           'Default'  => \&auth_user_screen,
           'ChangePassword' => \&change_pass,
	   'Help'     => \&help,
           );




# we dispatch to the appropriate routine
if ($CONFIG::WEB_NO_SERVICE) {
    print &error($CONFIG::WEB_NO_SERVICE_MSG);
} else {
    if (exists $States{$page}) {
        $States{$page}->();
    } else {
        print "No such page";
    }
}

&RatsStyle::end_cgi();
&RatsStyle::copyright_center();
&RatsStyle::doc_end();
exit;


sub auth_user_screen () {
    print 
	$q->h3("Hello, $auth_user. This page allows you to change your password."),
        $q->h3('Please enter the following info'),
        $q->start_form,
	$q->hidden(-name=>"Uname", -default=>$auth_user),
	$q->table({-border=>0,}, 
                  Tr({-align=>'TOP'},
                     [
                      td(["Firstname:\t", 
                          $q->textfield(-name=>'First'), 
			  "e.g. John"]),
                      td(["Lastname:\t", 
                          $q->textfield(-name=>'Last'),
			  "e.g. Smith"]),
                      td(["SSN:\t", 
                          $q->password_field(-name=>'SSN'), 
			  "9 digits, no dashes, e.g. 123456789"]),
                      td(["Birthday:\t", 
                          $q->password_field(-name=>'Bday'),
                          "month day year as MMDDYYYY, e.g. 01301980"]),
                      td(["New password:\t", 
                          $q->password_field(-name=>'Pass1'),
                          "Please consult the help screen for format, 
			  under the Password section."]),
                      td(["New password again:\t", 
                          $q->password_field(-name=>'Pass2'),
                          "Just to verify that you typed it right."]),
                      ]
                     )
                  ),
		      &to_page("ChangePassword");

    print $q->end_form;
    return;
}

sub to_page ($) {
    my ($val) = @_[0];
    return &RatsStyle::to_page($q,$val);
}


sub change_pass () {
    my ($first, $last, $ssn, $bday, $username, $pass1, $pass2);
    my ($bad_info, $pass_error, $old_pwent, $code, @data); 
    my ($response,$pass,$uid,$gid,$gcos,$home,$shell,$new_shell);
    my ($old_shent,$realm,$do_kerb,$timestamp1, $timestamp2);
    $first = $q->param('First');
    $last = $q->param('Last');
    $ssn = $q->param('SSN');
    $bday = $q->param('Bday');
    $username = $q->param('Uname');
    $pass1 = $q->param('Pass1');
    $pass2 = $q->param('Pass2');
    $realm = uc $CONFIG::KERBSHELL_REALM;

    ##make sure that the info is rational
    $bad_info = "";
    #check first name
    chomp $first; $first =~ s/^\s*//; $first =~ s/\s*$//;
    unless ($first =~ /^[A-Za-z]/) {
        $bad_info .= "Your first name was not entered in a recognizable format.<p>"; 
    }
    #check last name
    chomp $last; $last =~ s/^\s*//; $last =~ s/\s*$//;
    unless ($last =~ /^[A-Za-z]/) {
        $bad_info .= "Your last name was not entered in a recognizable format.<p>";
    }
    #check ssn
    chomp $ssn; $ssn =~ s/^\s*//; $ssn =~ s/\s*$//;
    unless ($ssn =~ /^\d{9}$/) {
        $bad_info .= "Your Social Security Number was not entered in the required format.<p>
";
    }
    #check bday
    chomp $bday; $bday =~ s/^\s*//; $bday =~ s/\s*$//;
    unless ($bday =~ /^\d{8}$/) {
        $bad_info .= "Your birthdate was not entered in the required format.<p>";
    }


    # check that the user exists and allowed to change pass
    $old_pwent = &$do('GETPWENT', $username, $target_host,$c_port);
    ($code, @data) = split (":", $old_pwent);
    if ($code == $CONFIG::API{'R_GETPWENT'}) {
        ($username,$pass,$uid,$gid,$gcos,$home,$shell,$timestamp1) = @data;

	# get the shadow file for the PRINC
	$old_shent = &$do('GETSHENT', $username, $target_host,$c_port);
	($code, @data) = split (":", $old_shent);
	if ($code == $CONFIG::API{'R_GETSHENT'}) {
	    $pass = $data[1];
	} else {
	    $bad_info .= "Username $username does not exist on this host.<p>";
	}
	# check if the user has a kerberos password
	if ($pass =~ /^$CONFIG::ENIGMA_TOKEN/) { 
	    $bad_info .= "User $username does not have a kerberos passwd!<p>";
	} elsif ($pass =~ /^$CONFIG::KERB_TOKEN/) { 
	    $pass =~ s/\#\#//;
	    if ($pass =~ /^[A-Z]/) { # for IID's we use +'s 
		$pass = "+$pass+";
	    }
	    $do_kerb = 1;
	} else {
	    $do_kerb = 0;
	}
    } else {
	$bad_info .= "Username $username does not exits on this host.<p>";
    } 


    
    #check that passwords match
    if ($pass1 ne $pass2) {
	$bad_info .= "The two passwords don't match!<p>";
    }
    
    # check new password format
    $pass_error = &check_pass($pass1);
    unless ($pass_error eq "") {
	$bad_info .= "$pass_error<p>";
    }


    #if any of the data was wrong ask for resubmit
    if ($bad_info) {
        print 
            error($CONFIG::WEB_DATA_ERROR),
            error($bad_info);
        return &auth_user_screen();
    }

    # change the password
    if ($do_kerb) {
	$response = &$do('CHNGPWD', $first,$last,$ssn,$bday,$pass,$realm
			 ,$pass1,$rats_master,$s_port);
	($code, @data) = split (":", $response);
	if ($code == $CONFIG::API{'RETCHNGPWD'}) {
	    if ($data[0] > 0 ) { 
		&$log("changed pass for $pass\n");
		print info("Password changed successfully!"),p;
		return;
	      } else {
		&$log("failed changed pass for $pass: @data\n");
		print error("Failed to perform kerb passwd change: @data"),p;
		return &auth_user_screen;
	      }
	} else {
	    &$log("failed chg pass for $pass by $first $last: @data\n");
	    if ($code == $CONFIG::API{'NOTFOUND'}) {
	      print error("Failed kerberos password change: You may have enterted your personal information incorrectly.");
	    } elsif ($data[0] eq '4011') {
	      print error($CONFIG::KERB_4011_TXT);
	    } else {
	      print error("Failed kerberos password change: @data");
	    }
	    return &auth_user_screen;
	}
    } else {
      if (uc($pass) eq 'BECOME') {
           print error("Passwords can not be changed for become accounts!");
           return;
       }

	# do crypt pass change.
	my $salt = substr $username, 0, 2;
	my $real_pass = crypt($pass, $salt);    
	
	# update password file
	$response = &$do('CHGPASSPW', $username, "x", 0, 
			 $target_host,$c_port);
	($code, @data) = split (":", $response);
	unless ($code == $CONFIG::API{'R_CHGPASSPW'}) {
	    print error("Failed to update password file: @data"),p;
	    return &auth_user_screen;
	}

	# update shadow file
	$response = &$do('CHGSPW', $username, $real_pass, 0, 
			 $target_host,$c_port);
	($code, @data) = split (":", $response);
	unless ($code == $CONFIG::API{'R_CHGSPW'}) {
	    print error("Failed to change shadow file: @data"),p;
	    return &auth_user_screen;
	}
	&$log("changed pass for $username\n");
	print info("Password changed successfully!"),p;
	return;
    }

}







sub help () {
    print $CONFIG::GENERAL_HELP;
    return;
}

sub close_window () {
    my $val = shift;
    return $q->submit(-NAME => ".State", -VALUE => $val, -onClick => 'window.close(self);');
}

sub back_window () {
    my $val = shift;
    return $q->submit(-NAME => ".State", -VALUE => $val, -onClick => 'history.back();');
}

# format error message
sub error ($) {
  return &RatsStyle::error($_[0]);
}

# format info message
sub info ($) {
  return &RatsStyle::info($_[0]);
}


# checks to see if a password is easy to guess
sub check_pass ($) {
    my $pass = @_[0];
    if (length($pass) < 6 ) {
	return "Password to short.(It must be between 6 and 8 characters long.)";
    }
    if (length($pass) > 8) {
	return "Password too long (It must be between 6 and 8 characters long.)";
    }

    my $count = 0;
    if ($pass =~ /[A-Z]/) {
	$count++;
    }
    if ($pass =~ /[a-z]/) {
	$count++;
    }
    if ($pass =~ /[0-9]/) {
	$count++;
    }
    if ($pass =~ /[\W_]/) {
	$count++;
    }
    if ($pass =~ /\s/) {
	return "Password is not allowed to contain blank spaces.";
    }
    if ($count < 3) {
	return "Password does not have enough character classes. (It must have at least 3 character classes.)";
    }
    return "";

}
