#!/usr/bin/perl -T
use File::Copy;
use strict;
use File::Basename;
use CGI qw(:standard);
use vars qw(%States $page);
use vars qw($user $auth_user $log $debug %VALID_SHELLS);
use vars qw($do $c_port $s_port $server $master);

require "/usr/local/accounts/lib/RatsClientLib.pm";
require "/usr/local/accounts/etc/rats_internal.conf";
require "/usr/local/accounts/lib/RatsClientStyle.pm";

# we define the log routine for all to see right here
$log = $CONFIG::COMMAND_HOOKS{'LOG'};
# same for debug
$debug = $CONFIG::COMMAND_HOOKS{'DEBUG'};

# remote call function
$do = \&RatsClientLib::send_remote_command;
$c_port=$CONFIG::CLIENT_PORT;
$s_port=$CONFIG::SERVER_PORT;
$server = $CONFIG::MASTER_HOST;
$master = $CONFIG::RATS_MASTER;


## the user gets set by httpd from the AUTH part
$auth_user=$ENV{REMOTE_USER};

## create a new query and set the header
my $q = new CGI;
print $q->header;
my $title = $CONFIG::CHGSHELL_WEB_TITLE;
print $q->start_html(-title=>$title,
		       -style=>{'src'=>$RatsStyle::StyleSheetLocation},
		       -dtd=>'-//W3C//DTD HTML 4.0 Transitional//EN',
		      );
  
&RatsStyle::doc_frame_nomenu($title);
&RatsStyle::start_cgi();


## And if the user is not set then we quit
if ($auth_user eq "") {
    print error("Unauthenticated. Bye.");
    &RatsStyle::end_cgi();
    &RatsStyle::copyright_center();
    &RatsStyle::doc_end();
    exit -1;
}

# read list of valid shells 
if (open(SFH, "<$CONFIG::ETC_SHELLS")) {
    while (<SFH>) {
	chomp;
	$VALID_SHELLS{$_} = 1;
    }
} else {
    print error("Failed to open $CONFIG::ETC_SHELLS. Bye.");
    &RatsStyle::end_cgi();
    &RatsStyle::copyright_center();
    &RatsStyle::doc_end();
    exit -1;
}



# this gets the current state of the script
$page = $q->param(".State") || "Default";


## This allows for the many in one script. 
%States = (
           'Default' => \&main_screen,
           'Change'  => \&lock_user,
	   );


# we dispatch to the appropriate routine
if ($CONFIG::WEB_NO_SERVICE) {
    print &error($CONFIG::WEB_NO_SERVICE_MSG);
} else {
    if (exists $States{$page}) {
	$States{$page}->();
    } else {
	print error("No such page");
    }
}
&RatsStyle::end_cgi();
&RatsStyle::copyright_center();
&RatsStyle::doc_end();
exit;


# give a user field and a pull down menu of locks
sub main_screen {
    my @list = keys %VALID_SHELLS;
    my ($username,$pass,$uid,$gid,$gcos,$home,$old_shell,$timestamp);

    # check user data
    my $response = &$do('GETPWENT', $auth_user, $server,$c_port);
    my ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_GETPWENT'}) {
	($username,$pass,$uid,$gid,$gcos,$home,$old_shell,$timestamp) = @data;
    } else {
	print error("No such user $username\n."),p;
	return;
    }

    # check if the current shell is valid.
    unless ($VALID_SHELLS{$old_shell}) {
	print error("You do not currently have a valid shell.\n");
	print error("You can only change shells if you have a valid one.\n");
	return;
    }


    print
        $q->h1("Hello, $auth_user. Choose the shell you want"),p,
        $q->start_form,
	"Shell:",
        $q->popup_menu(-name=>'SHELL',
                       -values=>[@list],
		       -default=>"$old_shell"),p,
	
        to_page("Change"),
        $q->end_form;
    return;
}


sub lock_user {
    my ($code,@data,$username,$pass,$uid,$gid,$gcos,$home,$old_shell);
    my ($response,$new_shell,$timestamp);
	
    my $shell = $q->param('SHELL');
    my $username =  $auth_user;
    
    # verify that lock type is valid 
    unless ($VALID_SHELLS{$shell}) {
	print error("Shell $shell is not valid."),p;
	return
    }

    # check user data
    $response = &$do('GETPWENT', $username, $server,$c_port);
    ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_GETPWENT'}) {
	($username,$pass,$uid,$gid,$gcos,$home,$old_shell,$timestamp) = @data;
    } else {
	print error("No such user $username\n."),p;
	return &main_screen();
    }

    # check if the current shell is valid.
    unless ($VALID_SHELLS{$old_shell}) {
	print error("You do not currently have a valid shell.\n");
	print error("You can only change shells if you have a valid one.\n");
	return &main_screen();
    }

    # check if the new shell is valid.
    unless ($VALID_SHELLS{$shell}) {
	print error("$shell is not a valid shell.\n");
	return &main_screen();
    }

    if ($old_shell eq $shell) {
	print 
	    info("Shell for $username left unchanged.\n"),
	    p;
	return;

    }

    # build the new shell
    $response = &$do('CHGSHELL', $username, $shell, $timestamp, $server,$c_port);
    ($code, @data) = split (":", $response);
    if ($code == $CONFIG::API{'R_CHGSHELL'}) { 
	print 
	    info("Shell for $username changed to \"$shell\"\n"),
	    p;
	return;
    } else {
	print error("Error setting new shell: @data\n"),p;
	return &main_screen();
    }
}

# submit button with a hidden field.
# its only arg should be the value of the hidden state
sub to_page ($) {
    my $val = shift;
    return &RatsStyle::to_page($q,$val);
}
# format error message
sub error ($) {
  return &RatsStyle::error($_[0]);
}

# format info message
sub info ($) {
  return &RatsStyle::info($_[0]);
}

